﻿using System;
using System.Collections.Generic;
using UnityEngine;

public class LevelManager : MonoBehaviour
{
    #region Fields
    [SerializeField]
    private GameObject energyOperatorUIPrefab;
    [SerializeField]
    private GameObject heightOperatorUIPrefab;
    [SerializeField]
    private GameObject lengthOperatorUIPrefab;
    [SerializeField]
    private GameObject phaseOperatorUIPrefab;
    [SerializeField]
    private List<Machine> machines;
    [SerializeField]
    private WaveAnimController waveAnimController;
    [SerializeField]
    private ScreenShake screenShake;

    public Action<Level> OnLevelAssigned;
    public Action<float> OnHightChanged;
    public Action<float> OnLengthChanged;
    public Action<float> OnPhaseChanged;
    public Action<float> OnTimeChanged;
	public Action<Interactions.Type> OnOrbDropped;

    private Level currentLevel;
    private int levelCount = 0;
    private float timePassedUntilTimeInterval = 0f;
    private float timeInterval = 0.1f;
    private float timePassed = 0f;
    #endregion

    #region Properties
    public Level CurrentLevel
    {
        get
        {
            return currentLevel;
        }
    }

    #endregion

    #region MonoBehaviour methods
    void Start()
    {
        levelCount = 0;
        AssignNewLevel();
    }

    void Update()
    {
        timePassedUntilTimeInterval += Time.deltaTime;
        timePassed += Time.deltaTime;

        if (timePassedUntilTimeInterval >= timeInterval)
        {
            timePassedUntilTimeInterval = 0f;

            waveAnimController.SetWaveLength(2 * timePassed / currentLevel.duration);
        }

        CheckLevelFinishedDueToTime();
    }
    #endregion

    #region Public methods
    public void DropOrb(Interactions.Type type)
    {
		if (OnOrbDropped != null) {
			OnOrbDropped (type);
		}

        switch (type)
        {
            case Interactions.Type.Height:
                currentLevel.IsHeightCompleted = true;
                break;

            case Interactions.Type.Length:
                currentLevel.IsLengthCompleted = true;
                break;

            case Interactions.Type.Phase:
                currentLevel.IsPhaseCompleted = true;
                break;

            case Interactions.Type.Energy:
                currentLevel.CompletedEnergyCount++;
                break;
        }

        CheckLevelFinished();
    }

    public void ChangeWaveHeight(float height)
    {
        currentLevel.enteredHeight = height;
        if (OnHightChanged != null)
        {
            OnHightChanged(height);
        }
    }

    public void ChangeWaveLength(float length)
    {
        currentLevel.enteredLength = length;
        if (OnLengthChanged != null)
        {
            OnLengthChanged(length);
        }
    }

    public void ChangeWavePhase(float phase)
    {
        currentLevel.enteredPhase = phase;
        if (OnPhaseChanged != null)
        {
            OnPhaseChanged(phase);
        }
    }
    #endregion

    #region Private methods
    private void AssignNewLevel()
    {
        levelCount++;

        float height = UnityEngine.Random.Range(0.1f, 1f);
        float length = UnityEngine.Random.Range(0.1f, 1f);
        float phase = UnityEngine.Random.Range(0.1f, 1f);
        int energyCount = UnityEngine.Random.Range(1, 4);
        float duration = GetDurationByLevelCount(levelCount);

        currentLevel = new Level(height, length, phase, energyCount, duration);

        if (OnLevelAssigned != null)
        {
            OnLevelAssigned(currentLevel);
        }

        AssignMachinesRandomTypes(energyCount);

        timePassed = 0f;
    }

    private void AssignMachinesRandomTypes(int energyCount)
    {
        // Assign None to all machines.
        foreach (Machine machine in machines)
        {
            machine.AssignInteractionType(Interactions.Type.None);
        }

        // Assign the height machine.
        int heightMachineIndex = UnityEngine.Random.Range(0, machines.Count);
        machines[heightMachineIndex].AssignInteractionType(Interactions.Type.Height);
        InstantiateHeightOperatorUI(machines[heightMachineIndex]);

        // Assign the length machine.
        int lengthMachineIndex = UnityEngine.Random.Range(0, machines.Count);
        while (lengthMachineIndex == heightMachineIndex)
        {
            lengthMachineIndex = UnityEngine.Random.Range(0, machines.Count);
        }
        machines[lengthMachineIndex].AssignInteractionType(Interactions.Type.Length);
        InstantiateLengthOperatorUI(machines[lengthMachineIndex]);

        // Assign the phase machine.
        int phaseMachineIndex = UnityEngine.Random.Range(0, machines.Count);
        while (phaseMachineIndex == heightMachineIndex || phaseMachineIndex == lengthMachineIndex)
        {
            phaseMachineIndex = UnityEngine.Random.Range(0, machines.Count);
        }
        machines[phaseMachineIndex].AssignInteractionType(Interactions.Type.Phase);
        InstantiatePhaseOperatorUI(machines[phaseMachineIndex]);

        // Assign the energy machines.
        for (int i = 0; i < energyCount; i++)
        {
            int energyMachineIndex = UnityEngine.Random.Range(0, machines.Count);
            if (machines[energyMachineIndex].Interactable.m_Interactions.m_Type == Interactions.Type.None)
            {
                machines[energyMachineIndex].AssignInteractionType(Interactions.Type.Energy);
                InstantiateEnergyOperatorUI(machines[energyMachineIndex]);
            }
            else
            {
                i--;
            }
        }
    }

    private void CheckLevelFinished()
    {
        if (currentLevel.IsHeightCompleted == true
            && currentLevel.IsLengthCompleted == true
            && currentLevel.IsPhaseCompleted == true
            && currentLevel.CompletedEnergyCount == currentLevel.EnergyCount)
        {
            AssignNewLevel();
        }
    }

    private void InstantiateEnergyOperatorUI(Machine machine)
    {
        GameObject energyOperatorUIGO = Instantiate(energyOperatorUIPrefab, machine.transform);
        energyOperatorUIGO.transform.localPosition = Vector3.zero;
    }

    private void InstantiateHeightOperatorUI(Machine machine)
    {
        GameObject heightOperatorUIGO = Instantiate(heightOperatorUIPrefab, machine.transform);
        heightOperatorUIGO.transform.localPosition = Vector3.zero;
    }

    private void InstantiateLengthOperatorUI(Machine machine)
    {
        GameObject lengthOperatorUIGO = Instantiate(lengthOperatorUIPrefab, machine.transform);
        lengthOperatorUIGO.transform.localPosition = Vector3.zero;
    }

    private void InstantiatePhaseOperatorUI(Machine machine)
    {
        GameObject phaseOperatorUIGO = Instantiate(phaseOperatorUIPrefab, machine.transform);
        phaseOperatorUIGO.transform.localPosition = Vector3.zero;
    }

    private float GetDurationByLevelCount(int levelCount)
    {
        float duration = 0f;

        switch (levelCount)
        {
            case 1:
                duration = 45f;
                break;

            case 2:
                duration = 40f;
                break;

            case 3:
                duration = 35f;
                break;

            case 4:
                duration = 30f;
                break;

            case 5:
                duration = 25f;
                break;

            case 6:
                duration = 20f;
                break;

            case 7:
                duration = 20f;
                break;

            case 8:
                duration = 15f;
                break;

            case 9:
                duration = 15f;
                break;

            case 10:
                duration = 10f;
                break;

            default:
                duration = 10f;
                break;
        }

        return duration;
    }

    private void CheckLevelFinishedDueToTime()
    {
        if (timePassed > currentLevel.duration)
        {
            // DAMAGE
            AssignNewLevel();
            screenShake.ShakeCamera(2f, 0.5f);
        }
    }
    #endregion
}