﻿using System.Collections.Generic;
using UnityEngine;

public class Machine : MonoBehaviour
{
    #region Fields
    [SerializeField]
    private LevelManager levelManager;
    [SerializeField]
    private Interactable interactable;
    [SerializeField]
    private SpriteRenderer spriteRenderer;
    [SerializeField]
    private GameObject heightButtonGO1;
    [SerializeField]
    private GameObject heightButtonGO2;
    [SerializeField]
    private GameObject energyButtonGO1;
    [SerializeField]
    private GameObject energyButtonGO2;
    [SerializeField]
    private List<Sprite> inactiveMachineSprites;
    [SerializeField]
    private Sprite energyMachineSprite;
    [SerializeField]
    private Sprite heightMachineSprite;
    [SerializeField]
    private Sprite lengthMachineSprite;
    [SerializeField]
    private Sprite phaseMachineSprite;

    private Operatable operatable;
    private bool isButtonGO1Visible;
    #endregion

    #region Properties
    public Interactable Interactable
    {
        get
        {
            return interactable;
        }
    }

    public Operatable Operatable
    {
        get
        {
            return operatable;
        }
    }
    #endregion

    #region MonoBehaviour methods
    void OnEnable()
    {
        interactable.OnValueChanged += OnValueChanged;
        interactable.OnStateChanged += OnStateChanged;
    }

    void OnDisable()
    {
        interactable.OnValueChanged -= OnValueChanged;
        interactable.OnStateChanged -= OnStateChanged;
    }

    void Update()
    {
        if (operatable != null)
        {
            operatable.OnUpdate();
        }
    }
    #endregion

    #region Public methods
    public void AssignInteractionType(Interactions.Type interactionType)
    {
        interactable.m_Interactions.SetType(interactionType);

        switch (interactionType)
        {
            case Interactions.Type.None:
                operatable = null;
                interactable.m_Interactions.SetState(Interactions.State.Inactive);
                spriteRenderer.sprite = inactiveMachineSprites[Random.Range(0, inactiveMachineSprites.Count)];
                heightButtonGO1.SetActive(false);
                heightButtonGO2.SetActive(false);
                energyButtonGO1.SetActive(false);
                energyButtonGO2.SetActive(false);
                break;

            case Interactions.Type.Height:
                operatable = new HeightOperatable(this);
                interactable.m_Interactions.SetState(Interactions.State.Waiting);
                spriteRenderer.sprite = heightMachineSprite;
                heightButtonGO1.SetActive(true);
                heightButtonGO2.SetActive(false);
                isButtonGO1Visible = true;
                energyButtonGO1.SetActive(false);
                energyButtonGO2.SetActive(false);
                break;

            case Interactions.Type.Length:
                operatable = new LengthOperatable(this);
                interactable.m_Interactions.SetState(Interactions.State.Waiting);
                spriteRenderer.sprite = lengthMachineSprite;
                heightButtonGO1.SetActive(false);
                heightButtonGO2.SetActive(false);
                energyButtonGO1.SetActive(false);
                energyButtonGO2.SetActive(false);
                break;

            case Interactions.Type.Phase:
                operatable = new PhaseOperatable(this);
                interactable.m_Interactions.SetState(Interactions.State.Waiting);
                spriteRenderer.sprite = phaseMachineSprite;
                heightButtonGO1.SetActive(false);
                heightButtonGO2.SetActive(false);
                energyButtonGO1.SetActive(false);
                energyButtonGO2.SetActive(false);
                break;

            case Interactions.Type.Energy:
                operatable = new EnergyOperatable(this);
                interactable.m_Interactions.SetState(Interactions.State.Waiting);
                spriteRenderer.sprite = energyMachineSprite;
                energyButtonGO1.SetActive(true);
                energyButtonGO2.SetActive(false);
                isButtonGO1Visible = true;
                heightButtonGO1.SetActive(false);
                heightButtonGO2.SetActive(false);
                break;
        }
    }

    public bool IsOperatable()
    {
        if (operatable == null)
        {
            return false;
        }

        if (interactable.m_Interactions.m_Type == Interactions.Type.None)
        {
            return false;
        }

        if (interactable.m_Interactions.m_State == Interactions.State.Active
            || interactable.m_Interactions.m_State == Interactions.State.Collected
            || interactable.m_Interactions.m_State == Interactions.State.Dropped
            || interactable.m_Interactions.m_State == Interactions.State.Inactive)
        {
            return false;
        }

        return true;
    }

    public bool IsCurrentlyOperating()
    {
        if (operatable == null)
        {
            return false;
        }

        if (interactable.m_Interactions.m_Type == Interactions.Type.None)
        {
            return false;
        }

        if (interactable.m_Interactions.m_State == Interactions.State.Waiting
            || interactable.m_Interactions.m_State == Interactions.State.Collected
            || interactable.m_Interactions.m_State == Interactions.State.Dropped
            || interactable.m_Interactions.m_State == Interactions.State.Inactive)
        {
            return false;
        }

        return true;
    }

    public void HideEnergyButtons()
    {
        energyButtonGO1.SetActive(false);
        energyButtonGO2.SetActive(false);
    }

    public void AnimateButtonClick()
    {
        ToggleButtonGOs();
    }

    private void OnValueChanged(float value, Interactions.Type type)
    {
        switch (type)
        {
            case Interactions.Type.Height:
                levelManager.ChangeWaveHeight(value);
                break;

            case Interactions.Type.Length:
                levelManager.ChangeWaveLength(value);
                break;

            case Interactions.Type.Phase:
                levelManager.ChangeWavePhase(value);
                break;
        }
    }

    private void OnStateChanged(Interactions.State state, Interactions.Type type)
    {
        if (state == Interactions.State.Collected)
        {
            energyButtonGO1.SetActive(false);
            energyButtonGO2.SetActive(false);
            heightButtonGO1.SetActive(false);
            heightButtonGO2.SetActive(false);
        }
    }

    private void ToggleButtonGOs()
    {
        if (interactable.m_Interactions.m_Type == Interactions.Type.Height)
        {
            if (isButtonGO1Visible == true)
            {
                heightButtonGO1.SetActive(false);
                heightButtonGO2.SetActive(true);
                isButtonGO1Visible = false;
            }
            else
            {
                heightButtonGO1.SetActive(true);
                heightButtonGO2.SetActive(false);
                isButtonGO1Visible = true;
            }
        }
        else if (interactable.m_Interactions.m_Type == Interactions.Type.Energy)
        {
            if (isButtonGO1Visible == true)
            {
                energyButtonGO1.SetActive(false);
                energyButtonGO2.SetActive(true);
                isButtonGO1Visible = false;
            }
            else
            {
                energyButtonGO1.SetActive(true);
                energyButtonGO2.SetActive(false);
                isButtonGO1Visible = true;
            }
        }
    }
    #endregion
}