﻿using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(Rigidbody2D))]
public class PlayerController : MonoBehaviour
{
    #region Fields
    [SerializeField]
    private LevelManager levelManager;
    [SerializeField]
    private PlayerInput playerInput;
    [SerializeField]
    private Animator animator;
    [SerializeField]
    private float speed;
    [SerializeField]
    private Transform orbParent;
    [SerializeField]
    private GameObject energyOrbPrefab;
    [SerializeField]
    private GameObject heightOrbPrefab;
    [SerializeField]
    private GameObject lengthOrbPrefab;
    [SerializeField]
    private GameObject phaseOrbPrefab;
    [SerializeField]
    private Transform visualParent;

    public List<SpriteRenderer> spriteRenderers;

    private new Rigidbody2D rigidbody2D;
    private PlayerState playerState;
    private Machine currentTargetMachine;
    private LauncherMachine launcherMachine;
    private bool isCarryingOrb = false;
    private Interactions.Type carryingOrbType;
    private GameObject orb;
    #endregion

    #region Properties
    public PlayerState State
    {
        get
        {
            return playerState;
        }
        set
        {
            playerState = value;
        }
    }
    #endregion

    #region MonoBehaviour methods
    void Awake()
    {
        rigidbody2D = GetComponent<Rigidbody2D>();
    }

    void Start()
    {
        State = PlayerState.Walking;
    }

    void Update()
    {
        if (transform.position.y < 3f)
        {
            
        }
        else
        {

        }
    }

    void OnEnable()
    {
        playerInput.OnLeftStickValueChanged += OnLeftStickValueChanged;
        playerInput.OnAButtonDown += OnAButtonDown;
        playerInput.OnXButtonDown += OnXButtonDown;
        levelManager.OnLevelAssigned += OnLevelAssigned;
    }

    void OnDisable()
    {
        playerInput.OnLeftStickValueChanged -= OnLeftStickValueChanged;
        playerInput.OnAButtonDown -= OnAButtonDown;
        playerInput.OnXButtonDown -= OnXButtonDown;
        levelManager.OnLevelAssigned -= OnLevelAssigned;
    }

    void OnTriggerEnter2D(Collider2D other)
    {
        MachineControlArea machineControlArea = other.GetComponent<MachineControlArea>();
        if (machineControlArea != null)
        {
            currentTargetMachine = machineControlArea.Machine;
            currentTargetMachine.Interactable.OnStateChanged += OnMachineStateChanged;
        }

        LauncherMachine launcherMachine = other.GetComponent<LauncherMachine>();
        if (launcherMachine != null)
        {
            this.launcherMachine = launcherMachine;
        }
    }

    void OnTriggerExit2D(Collider2D other)
    {
        MachineControlArea machineControlArea = other.GetComponent<MachineControlArea>();
        if (machineControlArea != null)
        {
            currentTargetMachine.Interactable.OnStateChanged -= OnMachineStateChanged;
            currentTargetMachine = null;
        }

        LauncherMachine launcherMachine = other.GetComponent<LauncherMachine>();
        if (launcherMachine != null)
        {
            this.launcherMachine = null;
        }
    }
    #endregion

    #region Callbacks
    private void OnLeftStickValueChanged(Vector2 value)
    {
        if (currentTargetMachine != null)
        {
            if (currentTargetMachine.Operatable != null && State == PlayerState.Operating)
            {
                currentTargetMachine.Operatable.OnLeftStickValueChanged(value);
            }
        }

        if (State != PlayerState.Walking)
        {
            TriggerAnimation("Operate");
            rigidbody2D.velocity = Vector2.zero;
            return;
        }

        rigidbody2D.velocity = value * speed;

        // Rotate right-left
        if (rigidbody2D.velocity.x > 0)
        {
            TriggerAnimation("Walk");
            visualParent.localScale = new Vector3(1f, 1f, 1f);
        }
        else if (rigidbody2D.velocity.x < 0)
        {
            TriggerAnimation("Walk");
            visualParent.localScale = new Vector3(-1f, 1f, 1f);
        }
        else
        {
            TriggerAnimation("Idle");
        }
    }

    private void OnAButtonDown()
    {
        if (currentTargetMachine != null && isCarryingOrb == false)
        {
            if (currentTargetMachine.Operatable != null)
            {
                currentTargetMachine.Operatable.OnAButtonDown();
            }
        }

        if (launcherMachine != null && isCarryingOrb == true)
        {
            DropOrb();
        }
    }

    private void OnXButtonDown()
    {
        if (currentTargetMachine != null && isCarryingOrb == false)
        {
            if (currentTargetMachine.Operatable != null)
            {
                TriggerAnimation("Operate");

                currentTargetMachine.Operatable.OnXButtonDown();
            }
        }
    }

    private void CarryOrb(Interactions.Type type)
    {
        carryingOrbType = type;
        isCarryingOrb = true;

        switch (type)
        {
            case Interactions.Type.Energy:
                orb = Instantiate(energyOrbPrefab, orbParent);
                break;

            case Interactions.Type.Height:
                orb = Instantiate(heightOrbPrefab, orbParent);
                break;

            case Interactions.Type.Length:
                orb = Instantiate(lengthOrbPrefab, orbParent);
                break;

            case Interactions.Type.Phase:
                orb = Instantiate(phaseOrbPrefab, orbParent);
                break;
        }

        orb.transform.localPosition = Vector2.zero;
    }

    private void DropOrb()
    {
        levelManager.DropOrb(carryingOrbType);
        isCarryingOrb = false;
        carryingOrbType = Interactions.Type.None;
        Destroy(orb);
    }

    private void TriggerAnimation(string name)
    {
        if (animator != null)
        {
            animator.SetTrigger(name);
        }
    }

    private void OnLevelAssigned(Level lvl)
    {
        isCarryingOrb = false;
        carryingOrbType = Interactions.Type.None;
        Destroy(orb);
    }
    #endregion

    #region Callbacks
    private void OnMachineStateChanged(Interactions.State state, Interactions.Type type)
    {
        if (state == Interactions.State.Collected)
        {
            CarryOrb(type);
            State = PlayerState.Walking;
        }
        else if (state == Interactions.State.Waiting)
        {
            State = PlayerState.Walking;
        }
        else if (state == Interactions.State.Active)
        {
            State = PlayerState.Operating;
        }
    }
    #endregion
}